from pathlib import Path

from pydantic import BaseModel

from coovts.types import api, event, get_api_response_model, get_message_type
from coovts.types.shared import get_event_name

FILE_HEAD = """\
from abc import ABC, abstractmethod
from collections.abc import Callable, Coroutine
from types import EllipsisType
from typing import Any, Literal, overload

from pydantic import BaseModel

from . import api, event

type _Deco[**P, R] = Callable[[Callable[P, R]], Callable[P, R]]
type _Co[T] = Coroutine[Any, Any, T]

class PluginAPI(ABC):
    @abstractmethod
    async def _call_api(
        self,
        data: Any,
        *,
        message_type: str | None = None,
        response_model: type[BaseModel] | None | EllipsisType = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> Any: ...
    @abstractmethod
    def _handle_event[T: BaseModel](
        self,
        event_data_model: type[T],
        event_name: str | None = None,
    ) -> _Deco[[T], _Co[Any]]: ...

    # region builtin apis
"""

API_TEMPLATE = """
    @overload
    async def call_api(
        self,
        data: api.{req},
        *,
        message_type: Literal["{msg_t}"] = ...,
        response_model: type[api.{resp}] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.{resp}: ..."""

API_REST = """

    # endregion

    @overload
    async def call_api[M: BaseModel](
        self,
        data: BaseModel,
        *,
        message_type: str | None = None,
        response_model: type[M],
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> M: ...
    @overload
    async def call_api(
        self,
        data: BaseModel,
        *,
        message_type: str | None = None,
        response_model: None = None,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> dict[str, Any]: ...
    # otherwise message_type is required
    @overload
    async def call_api[M: BaseModel](
        self,
        data: Any,
        *,
        message_type: str,
        response_model: type[M],
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> M: ...
    @overload
    async def call_api(
        self,
        data: Any,
        *,
        message_type: str,
        response_model: None = None,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> dict[str, Any]: ...
"""

EVENT_HEAD = """
    # region builtin events
"""

EVENT_TEMPLATE = """
    @overload
    def handle_event[T: event.{model}](
        self,
        event_data_model: type[T],
        event_name: Literal["{name}"] = ...,
    ) -> _Deco[[T], _Co[Any]]: ..."""

EVENT_TAIL = """

    # endregion

    @overload
    def handle_event[T: BaseModel](
        self,
        event_data_model: type[T],
        event_name: str | None = None,
    ) -> _Deco[[T], _Co[Any]]: ...
"""

PYI_PATH = Path(__file__).parent.parent / "coovts" / "types" / "plugin_api.pyi"

with PYI_PATH.open("w", encoding="u8") as f:
    f.write(FILE_HEAD)
    for name, model in api.__dict__.items():
        if not name.endswith("Request"):
            continue
        assert issubclass(model, BaseModel)
        f.write(
            API_TEMPLATE.format(
                req=model.__name__,
                msg_t=get_message_type(model),
                resp=get_api_response_model(model).__name__,
            ),
        )
    f.write(API_REST)

    f.write(EVENT_HEAD)
    for name, model in event.__dict__.items():
        if not name.endswith("EventData"):
            continue
        assert issubclass(model, BaseModel)
        f.write(
            EVENT_TEMPLATE.format(
                name=get_event_name(model),
                model=model.__name__,
            ),
        )
    f.write(EVENT_TAIL)



from typing import Annotated, Any

from cookit.pyd import model_with_model_config
from pydantic import BaseModel, ConfigDict, Field
from pydantic.alias_generators import to_camel

request_model_config = ConfigDict(
    alias_generator=to_camel,
    validate_by_alias=False,
    serialize_by_alias=True,
)
response_model_config = ConfigDict(
    alias_generator=to_camel,
)


with_request_model_config = model_with_model_config(request_model_config)
with_response_model_config = model_with_model_config(response_model_config)


@with_request_model_config
class BaseRequest(BaseModel):
    api_name: str = "VTubeStudioPublicAPI"
    api_version: str = "1.0"
    request_id: Annotated[str | None, Field(alias="requestID")] = None
    message_type: str
    data: Any


@with_response_model_config
class BaseResponse(BaseModel):
    api_name: str
    api_version: str = "1.0"
    timestamp: int
    """13 digits"""
    request_id: Annotated[str | None, Field(alias="requestID")]
    message_type: str
    data: Any


def get_api_response_model(model: type[BaseModel] | BaseModel) -> type[BaseModel]:
    if not isinstance(model, type):
        model = type(model)

    if resp_m := getattr(model, "resp_m", None):
        if issubclass(resp_m, BaseModel):
            return resp_m
        raise TypeError(f"Model's 'resp_m' should be a BaseModel, not {type(resp_m)}")

    from . import api

    if resp_t := getattr(model, "resp_t", None):
        if not isinstance(resp_t, str):
            raise TypeError(f"Model's 'resp_t' should be a str, not {type(resp_t)}")
        if resp_m := getattr(api, resp_t, None):
            return resp_m
        raise ValueError(f"Model's resp_t '{resp_t}' not found in api module")

    if model.__name__.endswith("Request"):
        resp_t = f"{model.__name__[:-7]}Response"
        if resp_t and (resp_m := getattr(api, resp_t, None)):
            return resp_m

    raise ValueError(
        f"Cannot find suitable response model for {model}, please define manually",
    )


def get_message_type(model: type[BaseModel] | BaseModel) -> str:
    if not isinstance(model, type):
        model = type(model)
    if msg_t := getattr(model, "msg_t", None):
        if isinstance(msg_t, str):
            return msg_t
        raise TypeError(f"Model's 'msg_t' should be a str, not {type(msg_t)}")
    return model.__name__


def get_event_name(model: type[BaseModel] | BaseModel) -> str:
    if not isinstance(model, type):
        model = type(model)
    if msg_t := getattr(model, "msg_t", None):
        if isinstance(msg_t, str):
            return msg_t
        raise TypeError(f"Model's 'msg_t' should be a str, not {type(msg_t)}")
    if model.__name__.endswith("EventData"):
        return model.__name__[:-4]
    raise ValueError(
        f"Cannot find suitable event name for {model}, please define manually",
    )



from asyncio import Future, wait_for
from dataclasses import dataclass, field
from typing import Any

from pydantic import BaseModel


@dataclass
class ResponseInfo[M: BaseModel]:
    model: type[M] | None = None
    future: Future[M] = field(default_factory=Future)


class RequestManager:
    def __init__(self, id_counter_max: int = 2147483647) -> None:
        self.id_counter_max = id_counter_max

        self.id_counter = 0
        self.id_signals: dict[str, ResponseInfo] = {}

    def has_id(self, req_id: str) -> bool:
        return req_id in self.id_signals

    def acquire_request(self, req_id: str, model: type[BaseModel] | None = None):
        if req_id in self.id_signals:
            raise RuntimeError("ID already in use")
        self.id_signals[req_id] = ResponseInfo(model=model)

    def acquire_next_request(self, model: type[BaseModel] | None = None) -> str:
        if self.id_counter >= self.id_counter_max:
            self.id_counter = 0
        self.id_counter += 1
        while (id_str := str(self.id_counter)) in self.id_signals:
            self.id_counter += 1
        self.acquire_request(id_str, model)
        return id_str

    async def wait_response(
        self,
        req_id: str,
        timeout: float | None = 30,  # noqa: ASYNC109
        pop: bool = True,
    ) -> Any:
        if timeout == 0:
            timeout = None
        try:
            return await wait_for(self.id_signals[req_id].future, timeout)
        finally:
            if pop and self.has_id(req_id):
                self.pop(req_id)

    def pop(self, req_id: str) -> ResponseInfo:
        return self.id_signals.pop(req_id)

    def reset(self) -> None:
        self.id_counter = 0
        signals = self.id_signals.copy()
        self.id_signals.clear()
        for signal in signals.values():
            signal.future.cancel()



from typing import Annotated

from pydantic import BaseModel, Field

from ..shared import with_request_model_config, with_response_model_config


@with_response_model_config
class ExpressionParameter(BaseModel):
    name: str
    value: float


@with_response_model_config
class HotkeyRef(BaseModel):
    name: str
    id: str


@with_response_model_config
class ExpressionInfo(BaseModel):
    name: str
    file: str
    active: bool
    deactivate_when_key_is_let_go: bool
    auto_deactivate_after_seconds: bool
    seconds_remaining: float
    used_in_hotkeys: list[HotkeyRef] = []
    parameters: list[ExpressionParameter] = []


@with_request_model_config
class ExpressionStateRequest(BaseModel):
    details: bool = True
    expression_file: str | None = None


@with_response_model_config
class ExpressionStateResponse(BaseModel):
    model_loaded: bool
    model_name: str
    model_id: Annotated[str, Field(alias="modelID")]
    expressions: list[ExpressionInfo]


@with_request_model_config
class ExpressionActivationRequest(BaseModel):
    expression_file: str
    fade_time: float = 0.25
    active: bool


@with_response_model_config
class ExpressionActivationResponse(BaseModel):
    pass



from pydantic import BaseModel

from ..shared import with_request_model_config, with_response_model_config


@with_response_model_config
class CapturePartColor(BaseModel):
    active: bool
    color_r: int
    color_g: int
    color_b: int


@with_request_model_config
class SceneColorOverlayInfoRequest(BaseModel):
    pass


@with_response_model_config
class SceneColorOverlayInfoResponse(BaseModel):
    active: bool
    items_included: bool
    is_window_capture: bool
    base_brightness: int
    color_boost: int
    smoothing: int
    color_overlay_r: int
    color_overlay_g: int
    color_overlay_b: int
    color_avg_r: int
    color_avg_g: int
    color_avg_b: int
    left_capture_part: CapturePartColor
    middle_capture_part: CapturePartColor
    right_capture_part: CapturePartColor



from .art_mesh import (
    ArtMeshListRequest as ArtMeshListRequest,
    ArtMeshListResponse as ArtMeshListResponse,
    ArtMeshMatcher as ArtMeshMatcher,
    ArtMeshSelectionRequest as ArtMeshSelectionRequest,
    ArtMeshSelectionResponse as ArtMeshSelectionResponse,
    ColorData as ColorData,
    ColorTintRequest as ColorTintRequest,
    ColorTintResponse as ColorTintResponse,
)
from .auth import (
    AuthenticationRequest as AuthenticationRequest,
    AuthenticationResponse as AuthenticationResponse,
    AuthenticationTokenRequest as AuthenticationTokenRequest,
    AuthenticationTokenResponse as AuthenticationTokenResponse,
)
from .error import (
    APIErrorResponse as APIErrorResponse,
)
from .event import (
    EventSubscriptionRequest as EventSubscriptionRequest,
    EventSubscriptionResponse as EventSubscriptionResponse,
)
from .expression import (
    ExpressionActivationRequest as ExpressionActivationRequest,
    ExpressionActivationResponse as ExpressionActivationResponse,
    ExpressionInfo as ExpressionInfo,
    ExpressionParameter as ExpressionParameter,
    ExpressionStateRequest as ExpressionStateRequest,
    ExpressionStateResponse as ExpressionStateResponse,
    HotkeyRef as HotkeyRef,
)
from .hotkey import (
    HotkeyInfo as HotkeyInfo,
    HotkeysInCurrentModelRequest as HotkeysInCurrentModelRequest,
    HotkeysInCurrentModelResponse as HotkeysInCurrentModelResponse,
    HotkeyTriggerRequest as HotkeyTriggerRequest,
    HotkeyTriggerResponse as HotkeyTriggerResponse,
)
from .info import (
    APIStateRequest as APIStateRequest,
    APIStateResponse as APIStateResponse,
    StatisticsRequest as StatisticsRequest,
    StatisticsResponse as StatisticsResponse,
    VTSFolderInfoRequest as VTSFolderInfoRequest,
    VTSFolderInfoResponse as VTSFolderInfoResponse,
)
from .item import (
    ItemAnimationControlRequest as ItemAnimationControlRequest,
    ItemAnimationControlResponse as ItemAnimationControlResponse,
    ItemFileInfo as ItemFileInfo,
    ItemInstanceInfo as ItemInstanceInfo,
    ItemListRequest as ItemListRequest,
    ItemListResponse as ItemListResponse,
    ItemLoadRequest as ItemLoadRequest,
    ItemLoadResponse as ItemLoadResponse,
    ItemMoveInfo as ItemMoveInfo,
    ItemMoveRequest as ItemMoveRequest,
    ItemMoveResponse as ItemMoveResponse,
    ItemMoveResult as ItemMoveResult,
    ItemPinInfo as ItemPinInfo,
    ItemPinRequest as ItemPinRequest,
    ItemPinResponse as ItemPinResponse,
    ItemUnloadRequest as ItemUnloadRequest,
    ItemUnloadResponse as ItemUnloadResponse,
    UnloadedItem as UnloadedItem,
)
from .model import (
    AvailableModelsRequest as AvailableModelsRequest,
    AvailableModelsResponse as AvailableModelsResponse,
    CurrentModelRequest as CurrentModelRequest,
    CurrentModelResponse as CurrentModelResponse,
    ModelInfo as ModelInfo,
    ModelLoadRequest as ModelLoadRequest,
    ModelLoadResponse as ModelLoadResponse,
    ModelPosition as ModelPosition,
    MoveModelRequest as MoveModelRequest,
    MoveModelResponse as MoveModelResponse,
)
from .ndi import (
    NDIConfigRequest as NDIConfigRequest,
    NDIConfigResponse as NDIConfigResponse,
)
from .param import (
    FaceFoundRequest as FaceFoundRequest,
    FaceFoundResponse as FaceFoundResponse,
    InjectParameterDataRequest as InjectParameterDataRequest,
    InjectParameterDataResponse as InjectParameterDataResponse,
    InputParameter as InputParameter,
    InputParameterListRequest as InputParameterListRequest,
    InputParameterListResponse as InputParameterListResponse,
    Live2DParameter as Live2DParameter,
    Live2DParameterListRequest as Live2DParameterListRequest,
    Live2DParameterListResponse as Live2DParameterListResponse,
    ParameterCreationRequest as ParameterCreationRequest,
    ParameterCreationResponse as ParameterCreationResponse,
    ParameterDeletionRequest as ParameterDeletionRequest,
    ParameterDeletionResponse as ParameterDeletionResponse,
    ParameterValue as ParameterValue,
    ParameterValueRequest as ParameterValueRequest,
    ParameterValueResponse as ParameterValueResponse,
)
from .physics import (
    GetCurrentModelPhysicsRequest as GetCurrentModelPhysicsRequest,
    GetCurrentModelPhysicsResponse as GetCurrentModelPhysicsResponse,
    PhysicsGroup as PhysicsGroup,
    PhysicsOverride as PhysicsOverride,
    SetCurrentModelPhysicsRequest as SetCurrentModelPhysicsRequest,
    SetCurrentModelPhysicsResponse as SetCurrentModelPhysicsResponse,
)
from .post_process import (
    ConfigValue as ConfigValue,
    EffectConfigEntry as EffectConfigEntry,
    PostProcessingEffect as PostProcessingEffect,
    PostProcessingListRequest as PostProcessingListRequest,
    PostProcessingListResponse as PostProcessingListResponse,
    PostProcessingUpdateRequest as PostProcessingUpdateRequest,
    PostProcessingUpdateResponse as PostProcessingUpdateResponse,
)
from .scene import (
    CapturePartColor as CapturePartColor,
    SceneColorOverlayInfoRequest as SceneColorOverlayInfoRequest,
    SceneColorOverlayInfoResponse as SceneColorOverlayInfoResponse,
)



from abc import ABC, abstractmethod
from collections.abc import Callable, Coroutine
from types import EllipsisType
from typing import Any, Literal, overload

from pydantic import BaseModel

from . import api, event

type _Deco[**P, R] = Callable[[Callable[P, R]], Callable[P, R]]
type _Co[T] = Coroutine[Any, Any, T]

class PluginAPI(ABC):
    @abstractmethod
    async def _call_api(
        self,
        data: Any,
        *,
        message_type: str | None = None,
        response_model: type[BaseModel] | None | EllipsisType = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> Any: ...
    @abstractmethod
    def _handle_event[T: BaseModel](
        self,
        event_data_model: type[T],
        event_name: str | None = None,
    ) -> _Deco[[T], _Co[Any]]: ...

    # region builtin apis

    @overload
    async def call_api(
        self,
        data: api.ArtMeshListRequest,
        *,
        message_type: Literal["ArtMeshListRequest"] = ...,
        response_model: type[api.ArtMeshListResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ArtMeshListResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ArtMeshSelectionRequest,
        *,
        message_type: Literal["ArtMeshSelectionRequest"] = ...,
        response_model: type[api.ArtMeshSelectionResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ArtMeshSelectionResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ColorTintRequest,
        *,
        message_type: Literal["ColorTintRequest"] = ...,
        response_model: type[api.ColorTintResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ColorTintResponse: ...
    @overload
    async def call_api(
        self,
        data: api.AuthenticationRequest,
        *,
        message_type: Literal["AuthenticationRequest"] = ...,
        response_model: type[api.AuthenticationResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.AuthenticationResponse: ...
    @overload
    async def call_api(
        self,
        data: api.AuthenticationTokenRequest,
        *,
        message_type: Literal["AuthenticationTokenRequest"] = ...,
        response_model: type[api.AuthenticationTokenResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.AuthenticationTokenResponse: ...
    @overload
    async def call_api(
        self,
        data: api.EventSubscriptionRequest,
        *,
        message_type: Literal["EventSubscriptionRequest"] = ...,
        response_model: type[api.EventSubscriptionResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.EventSubscriptionResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ExpressionActivationRequest,
        *,
        message_type: Literal["ExpressionActivationRequest"] = ...,
        response_model: type[api.ExpressionActivationResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ExpressionActivationResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ExpressionStateRequest,
        *,
        message_type: Literal["ExpressionStateRequest"] = ...,
        response_model: type[api.ExpressionStateResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ExpressionStateResponse: ...
    @overload
    async def call_api(
        self,
        data: api.HotkeysInCurrentModelRequest,
        *,
        message_type: Literal["HotkeysInCurrentModelRequest"] = ...,
        response_model: type[api.HotkeysInCurrentModelResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.HotkeysInCurrentModelResponse: ...
    @overload
    async def call_api(
        self,
        data: api.HotkeyTriggerRequest,
        *,
        message_type: Literal["HotkeyTriggerRequest"] = ...,
        response_model: type[api.HotkeyTriggerResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.HotkeyTriggerResponse: ...
    @overload
    async def call_api(
        self,
        data: api.APIStateRequest,
        *,
        message_type: Literal["APIStateRequest"] = ...,
        response_model: type[api.APIStateResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.APIStateResponse: ...
    @overload
    async def call_api(
        self,
        data: api.StatisticsRequest,
        *,
        message_type: Literal["StatisticsRequest"] = ...,
        response_model: type[api.StatisticsResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.StatisticsResponse: ...
    @overload
    async def call_api(
        self,
        data: api.VTSFolderInfoRequest,
        *,
        message_type: Literal["VTSFolderInfoRequest"] = ...,
        response_model: type[api.VTSFolderInfoResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.VTSFolderInfoResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ItemAnimationControlRequest,
        *,
        message_type: Literal["ItemAnimationControlRequest"] = ...,
        response_model: type[api.ItemAnimationControlResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ItemAnimationControlResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ItemListRequest,
        *,
        message_type: Literal["ItemListRequest"] = ...,
        response_model: type[api.ItemListResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ItemListResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ItemLoadRequest,
        *,
        message_type: Literal["ItemLoadRequest"] = ...,
        response_model: type[api.ItemLoadResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ItemLoadResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ItemMoveRequest,
        *,
        message_type: Literal["ItemMoveRequest"] = ...,
        response_model: type[api.ItemMoveResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ItemMoveResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ItemPinRequest,
        *,
        message_type: Literal["ItemPinRequest"] = ...,
        response_model: type[api.ItemPinResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ItemPinResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ItemUnloadRequest,
        *,
        message_type: Literal["ItemUnloadRequest"] = ...,
        response_model: type[api.ItemUnloadResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ItemUnloadResponse: ...
    @overload
    async def call_api(
        self,
        data: api.AvailableModelsRequest,
        *,
        message_type: Literal["AvailableModelsRequest"] = ...,
        response_model: type[api.AvailableModelsResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.AvailableModelsResponse: ...
    @overload
    async def call_api(
        self,
        data: api.CurrentModelRequest,
        *,
        message_type: Literal["CurrentModelRequest"] = ...,
        response_model: type[api.CurrentModelResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.CurrentModelResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ModelLoadRequest,
        *,
        message_type: Literal["ModelLoadRequest"] = ...,
        response_model: type[api.ModelLoadResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ModelLoadResponse: ...
    @overload
    async def call_api(
        self,
        data: api.MoveModelRequest,
        *,
        message_type: Literal["MoveModelRequest"] = ...,
        response_model: type[api.MoveModelResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.MoveModelResponse: ...
    @overload
    async def call_api(
        self,
        data: api.NDIConfigRequest,
        *,
        message_type: Literal["NDIConfigRequest"] = ...,
        response_model: type[api.NDIConfigResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.NDIConfigResponse: ...
    @overload
    async def call_api(
        self,
        data: api.FaceFoundRequest,
        *,
        message_type: Literal["FaceFoundRequest"] = ...,
        response_model: type[api.FaceFoundResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.FaceFoundResponse: ...
    @overload
    async def call_api(
        self,
        data: api.InjectParameterDataRequest,
        *,
        message_type: Literal["InjectParameterDataRequest"] = ...,
        response_model: type[api.InjectParameterDataResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.InjectParameterDataResponse: ...
    @overload
    async def call_api(
        self,
        data: api.InputParameterListRequest,
        *,
        message_type: Literal["InputParameterListRequest"] = ...,
        response_model: type[api.InputParameterListResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.InputParameterListResponse: ...
    @overload
    async def call_api(
        self,
        data: api.Live2DParameterListRequest,
        *,
        message_type: Literal["Live2DParameterListRequest"] = ...,
        response_model: type[api.Live2DParameterListResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.Live2DParameterListResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ParameterCreationRequest,
        *,
        message_type: Literal["ParameterCreationRequest"] = ...,
        response_model: type[api.ParameterCreationResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ParameterCreationResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ParameterDeletionRequest,
        *,
        message_type: Literal["ParameterDeletionRequest"] = ...,
        response_model: type[api.ParameterDeletionResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ParameterDeletionResponse: ...
    @overload
    async def call_api(
        self,
        data: api.ParameterValueRequest,
        *,
        message_type: Literal["ParameterValueRequest"] = ...,
        response_model: type[api.ParameterValueResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.ParameterValueResponse: ...
    @overload
    async def call_api(
        self,
        data: api.GetCurrentModelPhysicsRequest,
        *,
        message_type: Literal["GetCurrentModelPhysicsRequest"] = ...,
        response_model: type[api.GetCurrentModelPhysicsResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.GetCurrentModelPhysicsResponse: ...
    @overload
    async def call_api(
        self,
        data: api.SetCurrentModelPhysicsRequest,
        *,
        message_type: Literal["SetCurrentModelPhysicsRequest"] = ...,
        response_model: type[api.SetCurrentModelPhysicsResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.SetCurrentModelPhysicsResponse: ...
    @overload
    async def call_api(
        self,
        data: api.PostProcessingListRequest,
        *,
        message_type: Literal["PostProcessingListRequest"] = ...,
        response_model: type[api.PostProcessingListResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.PostProcessingListResponse: ...
    @overload
    async def call_api(
        self,
        data: api.PostProcessingUpdateRequest,
        *,
        message_type: Literal["PostProcessingUpdateRequest"] = ...,
        response_model: type[api.PostProcessingUpdateResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.PostProcessingUpdateResponse: ...
    @overload
    async def call_api(
        self,
        data: api.SceneColorOverlayInfoRequest,
        *,
        message_type: Literal["SceneColorOverlayInfoRequest"] = ...,
        response_model: type[api.SceneColorOverlayInfoResponse] = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> api.SceneColorOverlayInfoResponse: ...

    # endregion

    @overload
    async def call_api[M: BaseModel](
        self,
        data: BaseModel,
        *,
        message_type: str | None = None,
        response_model: type[M],
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> M: ...
    @overload
    async def call_api(
        self,
        data: BaseModel,
        *,
        message_type: str | None = None,
        response_model: None = None,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> dict[str, Any]: ...
    # otherwise message_type is required
    @overload
    async def call_api[M: BaseModel](
        self,
        data: Any,
        *,
        message_type: str,
        response_model: type[M],
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> M: ...
    @overload
    async def call_api(
        self,
        data: Any,
        *,
        message_type: str,
        response_model: None = None,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> dict[str, Any]: ...

    # region builtin events

    @overload
    def handle_event[T: event.HotkeyTriggeredEventData](
        self,
        event_data_model: type[T],
        event_name: Literal["HotkeyTriggered"] = ...,
    ) -> _Deco[[T], _Co[Any]]: ...
    @overload
    def handle_event[T: event.ItemEventData](
        self,
        event_data_model: type[T],
        event_name: Literal["Item"] = ...,
    ) -> _Deco[[T], _Co[Any]]: ...
    @overload
    def handle_event[T: event.Live2DCubismEditorConnectedEventData](
        self,
        event_data_model: type[T],
        event_name: Literal["Live2DCubismEditorConnected"] = ...,
    ) -> _Deco[[T], _Co[Any]]: ...
    @overload
    def handle_event[T: event.ModelClickedEventData](
        self,
        event_data_model: type[T],
        event_name: Literal["ModelClicked"] = ...,
    ) -> _Deco[[T], _Co[Any]]: ...
    @overload
    def handle_event[T: event.ModelLoadedEventData](
        self,
        event_data_model: type[T],
        event_name: Literal["ModelLoaded"] = ...,
    ) -> _Deco[[T], _Co[Any]]: ...
    @overload
    def handle_event[T: event.ModelMovedEventData](
        self,
        event_data_model: type[T],
        event_name: Literal["ModelMoved"] = ...,
    ) -> _Deco[[T], _Co[Any]]: ...
    @overload
    def handle_event[T: event.ModelOutlineEventData](
        self,
        event_data_model: type[T],
        event_name: Literal["ModelOutline"] = ...,
    ) -> _Deco[[T], _Co[Any]]: ...
    @overload
    def handle_event[T: event.TestEventData](
        self,
        event_data_model: type[T],
        event_name: Literal["Test"] = ...,
    ) -> _Deco[[T], _Co[Any]]: ...
    @overload
    def handle_event[T: event.TrackingStatusChangedEventData](
        self,
        event_data_model: type[T],
        event_name: Literal["TrackingStatusChanged"] = ...,
    ) -> _Deco[[T], _Co[Any]]: ...

    # endregion

    @overload
    def handle_event[T: BaseModel](
        self,
        event_data_model: type[T],
        event_name: str | None = None,
    ) -> _Deco[[T], _Co[Any]]: ...



from typing import Annotated

from pydantic import BaseModel, Field

from ..shared import with_request_model_config, with_response_model_config


@with_request_model_config
class TestEventConfig(BaseModel):
    test_message_for_event: str


@with_response_model_config
class TestEventData(BaseModel):
    your_test_message: str
    counter: int


@with_request_model_config
class ModelLoadedEventConfig(BaseModel):
    model_id: Annotated[list[str] | None, Field(alias="modelID")] = None


@with_response_model_config
class ModelLoadedEventData(BaseModel):
    model_loaded: bool
    model_name: str
    model_id: Annotated[str, Field(alias="modelID")]


@with_request_model_config
class TrackingStatusChangedEventConfig(BaseModel):
    pass


@with_response_model_config
class TrackingStatusChangedEventData(BaseModel):
    face_found: bool
    left_hand_found: bool
    right_hand_found: bool


@with_request_model_config
class HotkeyTriggeredEventConfig(BaseModel):
    only_for_action: str | None = None
    ignore_hotkeys_triggered_by_api: bool = False


@with_response_model_config
class HotkeyTriggeredEventData(BaseModel):
    hotkey_id: Annotated[str, Field(alias="hotkeyID")]
    hotkey_name: str
    hotkey_action: str
    hotkey_file: str
    hotkey_triggered_by_api: bool
    model_id: Annotated[str, Field(alias="modelID")]
    model_name: str
    is_live2d_item: bool


@with_response_model_config
class ModelPosition(BaseModel):
    position_x: float
    position_y: float
    size: float
    rotation: float


@with_request_model_config
class ModelMovedEventConfig(BaseModel):
    pass


@with_response_model_config
class ModelMovedEventData(BaseModel):
    model_id: Annotated[str, Field(alias="modelID")]
    model_name: str
    model_position: ModelPosition


@with_request_model_config
class ModelOutlineEventConfig(BaseModel):
    draw: bool = False


@with_response_model_config
class Point2D(BaseModel):
    x: float
    y: float


@with_response_model_config
class ModelOutlineEventData(BaseModel):
    model_name: str
    model_id: Annotated[str, Field(alias="modelID")]
    convex_hull: list[Point2D]
    convex_hull_center: Point2D
    window_size: Point2D


@with_response_model_config
class ArtMeshHitInfo(BaseModel):
    model_id: Annotated[str, Field(alias="modelID")]
    art_mesh_id: Annotated[str, Field(alias="artMeshID")]
    angle: float
    size: float
    vertex_id1: Annotated[int, Field(alias="vertexID1")]
    vertex_id2: Annotated[int, Field(alias="vertexID2")]
    vertex_id3: Annotated[int, Field(alias="vertexID3")]
    vertex_weight1: float
    vertex_weight2: float
    vertex_weight3: float


@with_response_model_config
class ArtMeshHit(BaseModel):
    art_mesh_order: int
    is_masked: bool
    hit_info: ArtMeshHitInfo


@with_request_model_config
class ModelClickedEventConfig(BaseModel):
    only_clicks_on_model: bool = True


@with_response_model_config
class ModelClickedEventData(BaseModel):
    model_loaded: bool
    loaded_model_id: Annotated[str, Field(alias="loadedModelID")]
    loaded_model_name: str
    model_was_clicked: bool
    mouse_button_id: Annotated[int, Field(alias="mouseButtonID")]
    click_position: Point2D
    window_size: Point2D
    clicked_art_mesh_count: int
    art_mesh_hits: list[ArtMeshHit]


@with_request_model_config
class ItemEventConfig(BaseModel):
    item_instance_ids: Annotated[
        list[str] | None,
        Field(alias="item_instance_ids"),
    ] = None
    item_file_names: list[str] | None = None


@with_response_model_config
class ItemEventData(BaseModel):
    item_event_type: str
    item_instance_id: Annotated[str, Field(alias="itemInstanceID")]
    item_file_name: str
    item_position: Point2D


@with_request_model_config
class Live2DCubismEditorConnectedEventConfig(BaseModel):
    pass


@with_response_model_config
class Live2DCubismEditorConnectedEventData(BaseModel):
    trying_to_connect: bool
    connected: bool
    should_send_parameters: bool



from typing import Annotated

from pydantic import BaseModel, Field

from ..shared import with_request_model_config, with_response_model_config


@with_request_model_config
class NDIConfigRequest(BaseModel):
    set_new_config: bool
    ndi_active: bool = True
    use_ndi5: bool = True
    use_custom_resolution: bool = True
    custom_width_ndi: Annotated[int, Field(alias="customWidthNDI")] = -1
    custom_height_ndi: Annotated[int, Field(alias="customHeightNDI")] = -1


@with_response_model_config
class NDIConfigResponse(BaseModel):
    set_new_config: bool
    ndi_active: bool
    use_ndi5: bool
    use_custom_resolution: bool
    custom_width_ndi: Annotated[int, Field(alias="customWidthNDI")]
    custom_height_ndi: Annotated[int, Field(alias="customHeightNDI")]



from abc import ABC, abstractmethod


class PluginAPI(ABC):
    @abstractmethod
    def _handle_event(self, *args, **kwargs): ...

    def handle_event(self, *args, **kwargs):
        return self._handle_event(*args, **kwargs)

    @abstractmethod
    async def _call_api(self, *args, **kwargs): ...

    def call_api(self, *args, **kwargs):
        return self._call_api(*args, **kwargs)



from pydantic import BaseModel

from ..shared import with_request_model_config, with_response_model_config


@with_request_model_config
class ConfigValue(BaseModel):
    config_id: str
    config_value: str


@with_response_model_config
class EffectConfigEntry(BaseModel):
    internal_id: str
    enum_id: str
    explanation: str
    type: str
    activation_config: bool
    float_value: float
    float_min: float
    float_max: float
    float_default: float
    int_value: int
    int_min: int
    int_max: int
    int_default: int
    color_value: str
    color_default: str
    color_has_alpha: bool
    bool_value: bool
    bool_default: bool
    string_value: str
    string_default: str
    scene_item_value: str
    scene_item_default: str


@with_response_model_config
class PostProcessingEffect(BaseModel):
    internal_id: str
    enum_id: str
    explanation: str
    effect_is_active: bool
    effect_is_restricted: bool
    config_entries: list[EffectConfigEntry]


@with_request_model_config
class PostProcessingListRequest(BaseModel):
    fill_post_processing_presets_array: bool = True
    fill_post_processing_effects_array: bool = True
    effect_id_filter: list[str] = []


@with_response_model_config
class PostProcessingListResponse(BaseModel):
    post_processing_supported: bool
    post_processing_active: bool
    can_send_post_processing_update_request_right_now: bool
    restricted_effects_allowed: bool
    preset_is_active: bool
    active_preset: str
    preset_count: int
    active_effect_count: int
    effect_count_before_filter: int
    config_count_before_filter: int
    effect_count_after_filter: int
    config_count_after_filter: int
    post_processing_effects: list[PostProcessingEffect] = []
    post_processing_presets: list[str] = []


@with_request_model_config
class PostProcessingUpdateRequest(BaseModel):
    post_processing_on: bool
    set_post_processing_preset: bool = False
    set_post_processing_values: bool = False
    preset_to_set: str = ""
    post_processing_fade_time: float = 0.0
    set_all_other_values_to_default: bool = True
    using_restricted_effects: bool = False
    randomize_all: bool = False
    randomize_all_chaos_level: float = 0.0
    post_processing_values: list[ConfigValue] = []


@with_response_model_config
class PostProcessingUpdateResponse(BaseModel):
    post_processing_active: bool
    preset_is_active: bool
    active_preset: str
    active_effect_count: int



import asyncio
import sys
from pathlib import Path

from coovts.plugin import Plugin
from coovts.types import api, event, get_event_name
from loguru import logger

AUTH_TOKEN_FILE = Path(__file__).parent / "auth_token.txt"
plugin = Plugin(
    "Example Plugin",
    "LgCookie",
    Path(__file__).parent / "icon.png",
    authentication_token=(
        AUTH_TOKEN_FILE.read_text().strip() if AUTH_TOKEN_FILE.exists() else None
    ),
)

# region loggings and auth token handle


@plugin.on_connecting
async def _():
    logger.info(f"Connecting to VTS WS at {plugin.endpoint}")


@plugin.on_connected
async def _():
    logger.info("Connected to VTS WS, authenticating")


@plugin.on_connect_failed
async def _(e: Exception):
    logger.error(
        f"Failed to connect, will retry after {plugin.reconnect_delay} seconds"
        f": {type(e).__name__}: {e}",
    )


@plugin.on_connection_closed
async def _(e: Exception):
    if plugin.stopped:
        return
    logger.opt(exception=e).error(
        f"Connection closed, will retry after {plugin.reconnect_delay} seconds",
    )


@plugin.on_parse_data_error
async def _(_raw: str | bytes, e: Exception):
    logger.opt(exception=e).error("Failed to parse data from VTS")


@plugin.on_authentication_token_got
async def _(token: str):
    logger.success("Authentication token got")
    AUTH_TOKEN_FILE.write_text(token)


@plugin.on_authenticated
async def _():
    logger.success("Authenticated successfully!")


@plugin.on_authenticate_failed
async def _(e: Exception):
    logger.error(f"Authentication failed: {type(e).__name__}: {e}")
    AUTH_TOKEN_FILE.unlink(missing_ok=True)
    logger.warning("Cleared authentication token, please re-authenticate at next retry")


@plugin.on_handler_run_failed
async def _(e: Exception):
    logger.opt(exception=e).error("Failed to run handler")


# endregion


# region api calling (event registering) & handling


@plugin.on_authenticated
async def _():
    await plugin.call_api(
        api.EventSubscriptionRequest(
            event_name=get_event_name(event.ModelMovedEventData),
            subscribe=True,
            config=event.ModelMovedEventConfig(),
        ),
    )
    logger.success("Subscribed to ModelMovedEvent")


@plugin.handle_event(event.ModelMovedEventData)
async def _(data: event.ModelMovedEventData):
    logger.info(f"Model moved: {data}")


# endregion


async def main() -> int:
    await plugin.run()
    return 0


if __name__ == "__main__":
    sys.exit(asyncio.run(main()))



import asyncio
from asyncio import AbstractEventLoop
from collections.abc import Callable
from functools import partial, wraps
from types import CoroutineType
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from concurrent.futures import Executor

type C[T] = CoroutineType[Any, Any, T]


def run_sync[**P, R](
    call: Callable[P, R],
    loop: AbstractEventLoop | None = None,
    executor: "Executor | None" = None,
) -> Callable[P, C[R]]:
    @wraps(call)
    async def _wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
        _loop = loop or asyncio.get_running_loop()
        return await _loop.run_in_executor(
            executor,
            partial(call, *args, **kwargs),
        )

    return _wrapper



from typing import Any

from pydantic import BaseModel

from ..shared import with_request_model_config, with_response_model_config


@with_request_model_config
class EventSubscriptionRequest(BaseModel):
    event_name: str
    subscribe: bool
    config: Any


@with_response_model_config
class EventSubscriptionResponse(BaseModel):
    subscribed_event_count: int
    subscribed_events: list[str]



from typing import Annotated

from pydantic import BaseModel, Field

from ..shared import with_request_model_config, with_response_model_config


@with_response_model_config
class ModelPosition(BaseModel):
    position_x: float
    position_y: float
    rotation: float
    size: float


@with_response_model_config
class ModelInfo(BaseModel):
    model_loaded: bool
    model_name: str
    model_id: Annotated[str, Field(alias="modelID")]
    vts_model_name: str
    vts_model_icon_name: str


@with_request_model_config
class CurrentModelRequest(BaseModel):
    pass


@with_response_model_config
class CurrentModelResponse(BaseModel):
    model_loaded: bool
    model_name: str
    model_id: Annotated[str, Field(alias="modelID")]
    vts_model_name: str
    vts_model_icon_name: str
    live2d_model_name: str
    model_load_time: int  # milliseconds
    time_since_model_loaded: int  # milliseconds
    number_of_live2d_parameters: int
    number_of_live2d_artmeshes: int
    has_physics_file: bool
    number_of_textures: int
    texture_resolution: int
    model_position: ModelPosition


@with_request_model_config
class AvailableModelsRequest(BaseModel):
    pass


@with_response_model_config
class AvailableModelsResponse(BaseModel):
    number_of_models: int
    available_models: list[ModelInfo]


@with_request_model_config
class ModelLoadRequest(BaseModel):
    model_id: Annotated[str, Field(alias="modelID")]


@with_response_model_config
class ModelLoadResponse(BaseModel):
    model_id: Annotated[str, Field(alias="modelID")]


@with_request_model_config
class MoveModelRequest(BaseModel):
    time_in_seconds: float
    values_are_relative_to_model: bool
    position_x: float | None = None
    position_y: float | None = None
    rotation: float | None = None
    size: float | None = None


@with_response_model_config
class MoveModelResponse(BaseModel):
    pass



from typing import Annotated

from pydantic import BaseModel, Field

from ..shared import with_response_model_config


@with_response_model_config
class APIErrorResponse(BaseModel):
    error_id: Annotated[int, Field(alias="errorID")]
    message: str



from typing import Annotated, Literal

from pydantic import BaseModel, Field

from ..shared import with_request_model_config, with_response_model_config

type FadeMode = Literal[
    "linear",
    "easeIn",
    "easeOut",
    "easeBoth",
    "overshoot",
    "zip",
]
type AngleRelativeTo = Literal[
    "RelativeToWorld",
    "RelativeToCurrentItemRotation",
    "RelativeToModel",
    "RelativeToPinPosition",
]
type SizeRelativeTo = Literal["RelativeToWorld", "RelativeToCurrentItemSize"]
type VertexPinType = Literal["Provided", "Center", "Random"]


@with_request_model_config
class ItemPinInfo(BaseModel):
    model_id: Annotated[str, Field(alias="modelID")] = ""
    art_mesh_id: Annotated[str, Field(alias="artMeshID")] = ""
    angle: float
    size: float
    vertex_id1: int = 0
    vertex_id2: int = 0
    vertex_id3: int = 0
    vertex_weight1: float = 0
    vertex_weight2: float = 0
    vertex_weight3: float = 0


@with_response_model_config
class ItemInstanceInfo(BaseModel):
    file_name: str
    instance_id: Annotated[str, Field(alias="instanceID")]
    order: int
    type: str
    censored: bool
    flipped: bool
    locked: bool
    smoothing: float
    framerate: float
    frame_count: int
    current_frame: int
    pinned_to_model: bool
    pinned_model_id: Annotated[str, Field(alias="pinnedModelID")]
    pinned_art_mesh_id: Annotated[str, Field(alias="pinnedArtMeshID")]
    group_name: str
    scene_name: str
    from_workshop: bool


@with_response_model_config
class ItemFileInfo(BaseModel):
    file_name: str
    type: str
    loaded_count: int


@with_response_model_config
class UnloadedItem(BaseModel):
    instance_id: Annotated[str, Field(alias="instanceID")]
    file_name: str


@with_response_model_config
class ItemMoveResult(BaseModel):
    item_instance_id: Annotated[str, Field(alias="itemInstanceID")]
    success: bool
    error_id: Annotated[int, Field(alias="errorID")]


@with_request_model_config
class ItemListRequest(BaseModel):
    include_available_spots: bool = False
    include_item_instances_in_scene: bool = False
    include_available_item_files: bool = False
    only_items_with_file_name: str | None = None
    only_items_with_instance_id: str | None = None


@with_response_model_config
class ItemListResponse(BaseModel):
    items_in_scene_count: int
    total_items_allowed_count: int
    can_load_items_right_now: bool
    available_spots: list[int] = []
    item_instances_in_scene: list[ItemInstanceInfo] = []
    available_item_files: list[ItemFileInfo] = []


@with_request_model_config
class ItemLoadRequest(BaseModel):
    file_name: str
    position_x: float
    position_y: float
    size: float
    rotation: float
    fade_time: float = 0.5
    order: int
    fail_if_order_taken: bool = False
    smoothing: float = 0
    censored: bool = False
    flipped: bool = False
    locked: bool = False
    unload_when_plugin_disconnects: bool = True
    custom_data_base64: str = ""
    custom_data_ask_user_first: bool = True
    custom_data_skip_asking_user_if_whitelisted: bool = True
    custom_data_ask_timer: float = -1


@with_response_model_config
class ItemLoadResponse(BaseModel):
    instance_id: Annotated[str, Field(alias="instanceID")]
    file_name: str

    class Response(BaseModel):
        instance_id: Annotated[str, Field(alias="instanceID")]
        file_name: str


@with_request_model_config
class ItemUnloadRequest(BaseModel):
    unload_all_in_scene: bool = False
    unload_all_loaded_by_this_plugin: bool = False
    allow_unloading_items_loaded_by_user_or_other_plugins: bool = True
    instance_ids: Annotated[list[str], Field(alias="instanceIDs")] = []
    file_names: list[str] = []


@with_response_model_config
class ItemUnloadResponse(BaseModel):
    unloaded_items: list[UnloadedItem]


@with_request_model_config
class ItemAnimationControlRequest(BaseModel):
    item_instance_id: Annotated[str, Field(alias="itemInstanceID")]
    framerate: float = -1
    frame: int = -1
    brightness: float = -1
    opacity: float = -1
    set_auto_stop_frames: bool = False
    auto_stop_frames: list[int] = []
    set_animation_play_state: bool = False
    animation_play_state: bool = True


@with_response_model_config
class ItemAnimationControlResponse(BaseModel):
    frame: int
    animation_playing: bool


class ItemMoveInfo(BaseModel):
    item_instance_id: Annotated[str, Field(alias="itemInstanceID")]
    time_in_seconds: float
    fade_mode: FadeMode = "linear"
    position_x: float = -1000
    position_y: float = -1000
    size: float = -1000
    rotation: float = -1000
    order: int = -1000
    set_flip: bool = False
    flip: bool = False
    user_can_stop: bool = True


@with_request_model_config
class ItemMoveRequest(BaseModel):
    items_to_move: list[ItemMoveInfo]


@with_response_model_config
class ItemMoveResponse(BaseModel):
    moved_items: list[ItemMoveResult]


@with_request_model_config
class ItemPinRequest(BaseModel):
    pin: bool
    item_instance_id: Annotated[str, Field(alias="itemInstanceID")]
    angle_relative_to: AngleRelativeTo = "RelativeToWorld"
    size_relative_to: SizeRelativeTo = "RelativeToWorld"
    vertex_pin_type: VertexPinType = "Center"
    pin_info: ItemPinInfo


@with_response_model_config
class ItemPinResponse(BaseModel):
    is_pinned: bool
    item_instance_id: Annotated[str, Field(alias="itemInstanceID")]
    item_file_name: str



from typing import Annotated

from pydantic import BaseModel, Field

from ..shared import with_request_model_config, with_response_model_config


@with_request_model_config
class PhysicsOverride(BaseModel):
    id: str
    value: float
    set_base_value: bool
    override_seconds: float


@with_response_model_config
class PhysicsGroup(BaseModel):
    group_id: Annotated[str, Field(alias="groupID")]
    group_name: str
    strength_multiplier: float
    wind_multiplier: float


@with_request_model_config
class GetCurrentModelPhysicsRequest(BaseModel):
    pass


@with_response_model_config
class GetCurrentModelPhysicsResponse(BaseModel):
    model_loaded: bool
    model_name: str
    model_id: Annotated[str, Field(alias="modelID")]
    model_has_physics: bool
    physics_switched_on: bool
    using_legacy_physics: bool
    physics_fps_setting: int
    base_strength: int
    base_wind: int
    api_physics_override_active: bool
    api_physics_override_plugin_name: str
    physics_groups: list[PhysicsGroup]


@with_request_model_config
class SetCurrentModelPhysicsRequest(BaseModel):
    strength_overrides: list[PhysicsOverride] = []
    wind_overrides: list[PhysicsOverride] = []


@with_response_model_config
class SetCurrentModelPhysicsResponse(BaseModel):
    pass



import asyncio
import base64
from collections.abc import Callable
from dataclasses import dataclass
from enum import Enum, auto
from pathlib import Path
from types import CoroutineType, EllipsisType
from typing import TYPE_CHECKING, Any

import websockets as ws
from cookit.loguru import warning_suppress
from pydantic import BaseModel

from .errors import APIError, AuthenticationFailedError, ValidationError
from .request import RequestManager
from .types import (
    BaseRequest,
    BaseResponse,
    get_api_response_model,
    get_event_name,
    get_message_type,
)
from .types.api import (
    APIErrorResponse,
    AuthenticationRequest,
    AuthenticationTokenRequest,
)
from .types.plugin_api import PluginAPI
from .utils import run_sync

if TYPE_CHECKING:
    from asyncio import Task

type C[T] = CoroutineType[Any, Any, T]

type ConnectingHandler = Callable[[], C[Any]]
type ConnectedHandler = Callable[[], C[Any]]
type ConnectFailedHandler = Callable[[Exception], C[Any]]
type ConnectionClosedHandler = Callable[[Exception], C[Any]]
type ParseDataErrorHandler = Callable[[str | bytes, Exception], C[Any]]
type AuthenticationTokenGotHandler = Callable[[str], C[Any]]
type AuthenticatedHandler = Callable[[], C[Any]]
type AuthenticationFailedHandler = Callable[[Exception], C[Any]]
type HandlerRunFailedHandler = Callable[[Exception], C[Any]]
type RecvRawHandler = Callable[[str | bytes], C[Any]]
type BeforeSendRawHandler = Callable[[str], C[Any]]

DEFAULT_ENDPOINT = "ws://localhost:8001"


@dataclass
class EventHandlerInfo[T: BaseModel]:
    model: type[T]
    handler: Callable[[T], Any]


def dispatch_handlers[**P, R](
    handlers: list[Callable[P, C[R]]],
    run_failed_handlers: list[HandlerRunFailedHandler] | None = None,
    *args: P.args,
    **kwargs: P.kwargs,
) -> list["Task[R | Exception]"] | None:
    async def run_task(f: Callable[P, C[R]]) -> R | Exception:
        try:
            return await f(*args, **kwargs)
        except Exception as e:
            if run_failed_handlers:
                dispatch_handlers(run_failed_handlers, None, e)
            return e

    return [asyncio.create_task(run_task(x)) for x in handlers]


class PluginState(Enum):
    STOPPED = auto()
    DISCONNECTED = auto()
    CONNECTING = auto()
    AUTHENTICATING = auto()
    AUTHENTICATED = auto()


class Plugin(PluginAPI):
    def __init__(
        self,
        plugin_name: str,
        plugin_developer: str,
        plugin_icon: str | bytes | Path | None = None,
        authentication_token: str | None = None,
        endpoint: str = DEFAULT_ENDPOINT,
        api_timeout: float | None = 30,
        reconnect_delay: float = 5,
    ) -> None:
        self.plugin_name = plugin_name
        self.plugin_developer = plugin_developer
        self.plugin_icon = self.prepare_icon(plugin_icon) if plugin_icon else None
        self.authentication_token = authentication_token
        self.endpoint = endpoint
        self.api_timeout = api_timeout
        self.reconnect_delay = reconnect_delay

        self.connecting_handlers: list[ConnectingHandler] = []
        self.connected_handlers: list[ConnectedHandler] = []
        self.connect_failed_handlers: list[ConnectFailedHandler] = []
        self.connection_closed_handlers: list[ConnectionClosedHandler] = []
        self.parse_data_error_handlers: list[ParseDataErrorHandler] = []
        self.authentication_token_got_handlers: list[AuthenticationTokenGotHandler] = []
        self.authenticated_handlers: list[AuthenticatedHandler] = []
        self.authenticate_failed_handlers: list[AuthenticationFailedHandler] = []
        self.handler_run_failed_handlers: list[HandlerRunFailedHandler] = []
        self.recv_raw_handlers: list[RecvRawHandler] = []
        self.before_send_raw_handlers: list[BeforeSendRawHandler] = []
        self.event_handlers: dict[str, list[EventHandlerInfo]] = {}

        self.client: ws.ClientConnection | None = None
        self.connecting = False
        self.authenticated = False
        self.stopped = True
        self.req_manager = RequestManager()

        self._recv_task: Task | None = None
        self._run_task: Task | None = None

    @staticmethod
    def prepare_icon(icon: str | bytes | Path) -> str:
        if isinstance(icon, Path):
            icon = icon.read_bytes()
        if isinstance(icon, bytes):
            icon = base64.b64encode(icon).decode()
        return icon

    @property
    def state(self) -> PluginState:
        if self.client:
            if self.authenticated:
                return PluginState.AUTHENTICATED
            return PluginState.AUTHENTICATING
        if self.connecting:
            return PluginState.CONNECTING
        return PluginState.DISCONNECTED

    def on_connecting[T: ConnectingHandler](self, handler: T) -> T:
        self.connecting_handlers.append(handler)
        return handler

    def on_connected[T: ConnectedHandler](self, handler: T) -> T:
        self.connected_handlers.append(handler)
        return handler

    def on_connect_failed[T: ConnectFailedHandler](self, handler: T) -> T:
        self.connect_failed_handlers.append(handler)
        return handler

    def on_connection_closed[T: ConnectionClosedHandler](self, handler: T) -> T:
        self.connection_closed_handlers.append(handler)
        return handler

    def on_parse_data_error[T: ParseDataErrorHandler](self, handler: T) -> T:
        self.parse_data_error_handlers.append(handler)
        return handler

    def on_authentication_token_got[T: AuthenticationTokenGotHandler](
        self,
        handler: T,
    ) -> T:
        self.authentication_token_got_handlers.append(handler)
        return handler

    def on_authenticated[T: AuthenticatedHandler](self, handler: T) -> T:
        self.authenticated_handlers.append(handler)
        return handler

    def on_authenticate_failed[T: AuthenticationFailedHandler](self, handler: T) -> T:
        self.authenticate_failed_handlers.append(handler)
        return handler

    def on_handler_run_failed[T: HandlerRunFailedHandler](self, handler: T) -> T:
        self.handler_run_failed_handlers.append(handler)
        return handler

    def on_recv_raw[T: RecvRawHandler](self, handler: T) -> T:
        self.recv_raw_handlers.append(handler)
        return handler

    def on_before_send_raw[T: BeforeSendRawHandler](self, handler: T) -> T:
        self.before_send_raw_handlers.append(handler)
        return handler

    def on_event[T: BaseModel, H: Callable[[BaseModel], Any]](
        self,
        event_name: str,
        model: type[T],
        handler: H,
    ) -> H:
        if event_name not in self.event_handlers:
            self.event_handlers[event_name] = []
        self.event_handlers[event_name].append(EventHandlerInfo(model, handler))
        return handler

    def _handle_event(
        self,
        event_data_model: type[BaseModel],
        event_name: str | None = None,
    ):
        if not event_name:
            event_name = get_event_name(event_data_model)

        def deco(f: Callable):
            return self.on_event(event_name, event_data_model, f)

        return deco

    def dispatch_handlers[**P, R](
        self,
        handlers: list[Callable[P, C[R]]],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> list["Task[R | Exception]"] | None:
        return dispatch_handlers(
            handlers,
            self.handler_run_failed_handlers,
            *args,
            **kwargs,
        )

    def ensure_client(self):
        if not self.client:
            raise RuntimeError("Client is not connected")
        return self.client

    async def _recv(self, client: ws.ClientConnection):
        raw = await client.recv()
        self.dispatch_handlers(self.recv_raw_handlers, raw)

        try:
            resp = BaseResponse.model_validate_json(raw)
        except Exception as e:
            self.dispatch_handlers(self.parse_data_error_handlers, raw, e)
            return

        instance_cache: dict[type[BaseModel], BaseModel] = {}

        def validate_data[T: BaseModel](
            model: type[T] | None = None,
        ) -> T | dict[str, Any]:
            if not model:
                return resp.data
            if model not in instance_cache:
                data = model.model_validate(resp.data)
                instance_cache[model] = data
            else:
                data = instance_cache[model]
            return data

        is_err = resp.message_type == APIError.message_type

        if resp.request_id and self.req_manager.has_id(resp.request_id):
            fut = self.req_manager.pop(resp.request_id)
            try:
                data: BaseModel | dict[str, Any] = validate_data(
                    APIErrorResponse if is_err else fut.model,
                )
            except Exception as e:
                err = ValidationError(raw, fut.model)
                err.__cause__ = e
                fut.future.set_exception(err)
            else:
                if is_err:
                    if TYPE_CHECKING:
                        assert isinstance(data, APIErrorResponse)
                    self.dispatch_handlers(
                        [run_sync(fut.future.set_exception)],
                        APIError(data),
                    )
                else:
                    fut.future.set_result(data)

        if resp.message_type in self.event_handlers:
            for handler_info in self.event_handlers[resp.message_type]:
                try:
                    data = validate_data(handler_info.model)
                except Exception as e:
                    self.dispatch_handlers(self.parse_data_error_handlers, raw, e)
                else:
                    self.dispatch_handlers([handler_info.handler], data)

    async def _recv_loop(self, client: ws.ClientConnection):
        while True:
            try:
                await self._recv(client)
            except Exception as e:
                self.client = None
                self.authenticated = False
                self.dispatch_handlers(self.connection_closed_handlers, e)
                await asyncio.sleep(self.reconnect_delay)
                break

    async def _disconnect(self):
        client = self.client
        task = self._recv_task
        self.client = None
        self._recv_task = None
        self.connecting = False
        self.authenticated = False
        self.req_manager.reset()
        self.req_manager = RequestManager()
        if client and (client.close_code is None):
            await client.close()
        if task and (not task.done()):
            task.cancel()

    async def reconnect(self):
        await self._disconnect()

        if self.connecting:
            raise RuntimeError("Already connecting")

        self.connecting = True
        self.dispatch_handlers(self.connecting_handlers)
        try:
            self.client = await ws.connect(self.endpoint)
        finally:
            self.connecting = False

        self.dispatch_handlers(self.connected_handlers)
        self._recv_task = asyncio.create_task(self._recv_loop(self.client))
        return self._recv_task

    async def stop(self):
        self.stopped = True
        await self._disconnect()
        if self._run_task:
            self._run_task.cancel()
        self._run_task = None

    async def _run(self):
        if not self.stopped:
            return
        self.stopped = False

        while not self.stopped:
            try:
                task = await self.reconnect()
            except Exception as e:
                if self.stopped:
                    break
                self.dispatch_handlers(self.connect_failed_handlers, e)
                await asyncio.sleep(self.reconnect_delay)
                continue

            try:
                await self.authenticate()
            except Exception as e:
                if self.stopped:
                    break
                self.dispatch_handlers(self.authenticate_failed_handlers, e)
                with warning_suppress("Disconnect failed"):
                    await self._disconnect()
                await asyncio.sleep(self.reconnect_delay)
                continue

            await task

    def run(self):
        if self._run_task and not self._run_task.done():
            raise RuntimeError("Already running")
        self._run_task = asyncio.create_task(self._run())
        return self._run_task

    async def _call_api(
        self,
        data: Any,
        *,
        message_type: str | None = None,
        response_model: type[BaseModel] | None | EllipsisType = ...,
        api_name: str = "VTubeStudioPublicAPI",
        api_version: str = "1.0",
        api_timeout: float | None | EllipsisType = ...,
    ) -> Any:
        req_timeout = self.api_timeout if api_timeout is ... else api_timeout

        if not message_type:
            if not isinstance(data, BaseModel):
                raise TypeError(
                    "'message_type' is required when 'data' is not a model",
                )
            message_type = get_message_type(data)

        if response_model is ...:
            if not isinstance(data, BaseModel):
                raise TypeError(
                    "'response_model' is required when 'data' is not a model",
                )
            response_model = get_api_response_model(data)

        client = self.ensure_client()
        req_id = self.req_manager.acquire_next_request(response_model)
        request = BaseRequest(
            api_name=api_name,
            api_version=api_version,
            request_id=req_id,
            message_type=message_type,
            data=data,
        )
        payload = request.model_dump_json()
        self.dispatch_handlers(self.before_send_raw_handlers, payload)
        await client.send(payload)
        return await self.req_manager.wait_response(req_id, req_timeout, pop=True)

    async def authenticate(self):
        if self.authenticated:
            return

        if not self.authentication_token:
            token_data = await self.call_api(
                AuthenticationTokenRequest(
                    plugin_name=self.plugin_name,
                    plugin_developer=self.plugin_developer,
                    plugin_icon=self.plugin_icon,
                ),
            )
            self.authentication_token = token_data.authentication_token
            self.dispatch_handlers(
                self.authentication_token_got_handlers,
                token_data.authentication_token,
            )

        data = await self.call_api(
            AuthenticationRequest(
                plugin_name=self.plugin_name,
                plugin_developer=self.plugin_developer,
                authentication_token=self.authentication_token,
            ),
        )
        if not data.authenticated:
            self.authentication_token = None
            raise AuthenticationFailedError(data)
        self.authenticated = True
        self.dispatch_handlers(self.authenticated_handlers)



from .event import (
    ArtMeshHit as ArtMeshHit,
    ArtMeshHitInfo as ArtMeshHitInfo,
    HotkeyTriggeredEventConfig as HotkeyTriggeredEventConfig,
    HotkeyTriggeredEventData as HotkeyTriggeredEventData,
    ItemEventConfig as ItemEventConfig,
    ItemEventData as ItemEventData,
    Live2DCubismEditorConnectedEventConfig as Live2DCubismEditorConnectedEventConfig,
    Live2DCubismEditorConnectedEventData as Live2DCubismEditorConnectedEventData,
    ModelClickedEventConfig as ModelClickedEventConfig,
    ModelClickedEventData as ModelClickedEventData,
    ModelLoadedEventConfig as ModelLoadedEventConfig,
    ModelLoadedEventData as ModelLoadedEventData,
    ModelMovedEventConfig as ModelMovedEventConfig,
    ModelMovedEventData as ModelMovedEventData,
    ModelOutlineEventConfig as ModelOutlineEventConfig,
    ModelOutlineEventData as ModelOutlineEventData,
    ModelPosition as ModelPosition,
    Point2D as Point2D,
    TestEventConfig as TestEventConfig,
    TestEventData as TestEventData,
    TrackingStatusChangedEventConfig as TrackingStatusChangedEventConfig,
    TrackingStatusChangedEventData as TrackingStatusChangedEventData,
)



from typing import Annotated

from pydantic import BaseModel, Field

from ..shared import with_request_model_config, with_response_model_config


@with_response_model_config
class HotkeyInfo(BaseModel):
    name: str
    type: str
    description: str
    file: str
    hotkey_id: Annotated[str, Field(alias="hotkeyID")]
    key_combination: list[str]
    on_screen_button_id: Annotated[int, Field(alias="onScreenButtonID")]


@with_request_model_config
class HotkeysInCurrentModelRequest(BaseModel):
    model_id: Annotated[str | None, Field(alias="modelID")] = None
    live2d_item_file_name: Annotated[
        str | None,
        Field(alias="live2DItemFileName"),
    ] = None


@with_response_model_config
class HotkeysInCurrentModelResponse(BaseModel):
    model_loaded: bool
    model_name: str
    model_id: Annotated[str, Field(alias="modelID")]
    available_hotkeys: list[HotkeyInfo]


@with_request_model_config
class HotkeyTriggerRequest(BaseModel):
    hotkey_id: Annotated[str, Field(alias="hotkeyID")]
    item_instance_id: Annotated[str | None, Field(alias="itemInstanceID")] = None


@with_response_model_config
class HotkeyTriggerResponse(BaseModel):
    hotkey_id: Annotated[str, Field(alias="hotkeyID")]



from . import api as api, event as event
from .shared import (
    BaseRequest as BaseRequest,
    BaseResponse as BaseResponse,
    get_api_response_model as get_api_response_model,
    get_event_name as get_event_name,
    get_message_type as get_message_type,
    request_model_config as request_model_config,
    response_model_config as response_model_config,
    with_request_model_config as with_request_model_config,
    with_response_model_config as with_response_model_config,
)



from typing import TYPE_CHECKING, ClassVar, override

from .types.api import APIErrorResponse, AuthenticationResponse

if TYPE_CHECKING:
    from pydantic import BaseModel


class VTSError(Exception):
    pass


class RequestError(VTSError):
    pass


class APIError(RequestError):
    message_type: ClassVar[str] = "APIError"

    def __init__(self, data: APIErrorResponse) -> None:
        self.data = data

    @override
    def __str__(self) -> str:
        return f"[{self.data.error_id}] {self.data.message}"


class NetworkError(RequestError):
    pass


class ValidationError(RequestError):
    def __init__(
        self,
        raw: str | bytes,
        model: type["BaseModel"] | None = None,
    ) -> None:
        self.raw = raw
        self.model = model

    def __str__(self) -> str:
        return f"Returned data cannot validate to {self.model and self.model.__name__}"


class AuthenticationFailedError(RequestError):
    def __init__(self, data: AuthenticationResponse) -> None:
        self.data = data

    @override
    def __str__(self) -> str:
        return self.data.reason



from typing import Annotated, Literal

from pydantic import BaseModel, Field

from ..shared import with_request_model_config, with_response_model_config

type ParameterMode = Literal["set", "add"]


@with_request_model_config
class ParameterValue(BaseModel):
    id: str
    value: float
    weight: float = 1.0


@with_response_model_config
class InputParameter(BaseModel):
    name: str
    added_by: str
    value: float
    min: float
    max: float
    default_value: float


@with_response_model_config
class Live2DParameter(BaseModel):
    name: str
    value: float
    min: float
    max: float
    default_value: float


@with_request_model_config
class FaceFoundRequest(BaseModel):
    pass


@with_response_model_config
class FaceFoundResponse(BaseModel):
    found: bool


@with_request_model_config
class InputParameterListRequest(BaseModel):
    pass


@with_response_model_config
class InputParameterListResponse(BaseModel):
    model_loaded: bool
    model_name: str
    model_id: Annotated[str, Field(alias="modelID")]
    custom_parameters: list[InputParameter]
    default_parameters: list[InputParameter]


@with_request_model_config
class ParameterValueRequest(BaseModel):
    name: str


@with_response_model_config
class ParameterValueResponse(InputParameter):
    pass


@with_request_model_config
class Live2DParameterListRequest(BaseModel):
    pass


@with_response_model_config
class Live2DParameterListResponse(BaseModel):
    model_loaded: bool
    model_name: str
    model_id: Annotated[str, Field(alias="modelID")]
    parameters: list[Live2DParameter]


@with_request_model_config
class ParameterCreationRequest(BaseModel):
    parameter_name: str
    explanation: str = ""
    min: float
    max: float
    default_value: float


@with_response_model_config
class ParameterCreationResponse(BaseModel):
    parameter_name: str


@with_request_model_config
class ParameterDeletionRequest(BaseModel):
    parameter_name: str


@with_response_model_config
class ParameterDeletionResponse(BaseModel):
    parameter_name: str


@with_request_model_config
class InjectParameterDataRequest(BaseModel):
    face_found: bool = False
    mode: ParameterMode = "set"
    parameter_values: list[ParameterValue]


@with_response_model_config
class InjectParameterDataResponse(BaseModel):
    pass



from pydantic import BaseModel

from ..shared import with_request_model_config, with_response_model_config


@with_request_model_config
class AuthenticationTokenRequest(BaseModel):
    plugin_name: str
    plugin_developer: str
    plugin_icon: str | None = None
    """128x128 PNG or JPG base64"""


@with_response_model_config
class AuthenticationTokenResponse(BaseModel):
    authentication_token: str


@with_request_model_config
class AuthenticationRequest(BaseModel):
    plugin_name: str
    plugin_developer: str
    authentication_token: str


@with_response_model_config
class AuthenticationResponse(BaseModel):
    authenticated: bool
    reason: str



from pydantic import BaseModel

from ..shared import with_request_model_config, with_response_model_config


@with_response_model_config
class ColorData(BaseModel):
    color_r: int
    color_g: int
    color_b: int
    color_a: int
    mix_with_scene_lighting_color: float = 1.0


@with_request_model_config
class ArtMeshMatcher(BaseModel):
    tint_all: bool = False
    art_mesh_number: list[int] = []
    name_exact: list[str] = []
    name_contains: list[str] = []
    tag_exact: list[str] = []
    tag_contains: list[str] = []


@with_request_model_config
class ArtMeshListRequest(BaseModel):
    pass


@with_response_model_config
class ArtMeshListResponse(BaseModel):
    model_loaded: bool
    number_of_art_mesh_names: int
    number_of_art_mesh_tags: int
    art_mesh_names: list[str]
    art_mesh_tags: list[str]


@with_request_model_config
class ColorTintRequest(BaseModel):
    color_tint: ColorData
    art_mesh_matcher: ArtMeshMatcher


@with_response_model_config
class ColorTintResponse(BaseModel):
    matched_art_meshes: int


@with_request_model_config
class ArtMeshSelectionRequest(BaseModel):
    text_override: str | None = None
    help_override: str | None = None
    requested_art_mesh_count: int = 0
    active_art_meshes: list[str] = []


@with_response_model_config
class ArtMeshSelectionResponse(BaseModel):
    success: bool
    active_art_meshes: list[str]
    inactive_art_meshes: list[str]



from pydantic import BaseModel

from ..shared import with_request_model_config, with_response_model_config


@with_request_model_config
class APIStateRequest(BaseModel):
    pass


@with_response_model_config
class APIStateResponse(BaseModel):
    active: bool
    v_tube_studio_version: str
    current_session_authenticated: bool


@with_request_model_config
class StatisticsRequest(BaseModel):
    pass


@with_response_model_config
class StatisticsResponse(BaseModel):
    uptime: int
    framerate: int
    v_tube_studio_version: str
    allowed_plugins: int
    connected_plugins: int
    started_with_steam: bool
    window_width: int
    window_height: int
    window_is_fullscreen: bool


@with_request_model_config
class VTSFolderInfoRequest(BaseModel):
    pass


@with_response_model_config
class VTSFolderInfoResponse(BaseModel):
    models: str
    backgrounds: str
    items: str
    config: str
    logs: str
    backup: str
